// -*- C++ -*-
/*!
 * @file  TerminalServer.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */
#include <iostream>
#include <sstream>
#include "Tools.h"
#ifdef _ALIVE_RTC
#include "AliveLib.h"
#endif
#include "TerminalServer.h"

#ifdef SCENARIO_FILE
SCEANRIO_CMD_MAP ScenarioCommand;
#endif

#define PA10_RH_CTRL	1
//#define _TEST_DATA		1

#ifdef PA10_RH_CTRL
	/**
	 * シナリオ実行ループ
	 */
	void *sceralio_running(void *param)
	{
		static TerminalServer*		pMain = (TerminalServer*)param;

		while(!(pMain->m_SystemEnd)){
			// シナリオ実行
			pMain->runScenario();
		}
		std::cout << "シナリオ実行ループ終了" << std::endl;
		return 0;
	}
#endif

void parse_gtalk_state(void *userData, const XML_Char *name, const XML_Char *atts[])
{
	bool			result, available = false;
	std::string		addr;
	TerminalServer	*pMain = (TerminalServer *)userData;
#ifdef _DEBUG2
	printf("[ELEMENT] %s Start!\n", name);
#endif
    for(int i = 0; ; i = i + 2){
    	if(atts[i] == NULL)	break;
#ifdef _DEBUG2
    	printf("[Attr] %s\n", atts[i]);
#endif
    	/*
    	 * 確認したところ、配列は以下のように格納されている
    	 * atts[0] = AttrName
    	 * atts[1] = AttrContent
    	 * atts[2] = AttrName
    	 * atts[3] = AttrContent
    	 *
    	 * すなわち、[0]:[1] [2]:[3] であり、属性名：値　が分解されて格納されている
    	 *
			[Attr] from
			[Attr] yotubata@fsi.co.jp/Talk.v105DA425169
			[Attr] state
			[Attr] available
    	 *
    	 */
    	if(strcmp(atts[i], "from") == 0){
    		// アドレス
    		std::string	addr_org = atts[i + 1];
    		addr = addr_org.substr(0, addr_org.find("/"));
    	}
    	if(strcmp(atts[i], "state") == 0){
    		// 状態
        	if(strcmp(atts[i+1], "unavailable") == 0){
        		available = false;
        	}
        	else{
        		available = true;
        	}
    	}
    }
#ifdef _DEBUG2
    std::cout << "e-mail = " << addr << std::endl;
    std::cout << "state = " << available << std::endl;
#endif
    result = pMain->SetMemberStatus(addr, available);
}

void parse_julius_result(void *userData, const XML_Char *name, const XML_Char *atts[])
{
	TerminalServer	*pMain = (TerminalServer *)userData;
	bool			result;
	std::string		candidateName;
	std::string		scoreStr;
	std::string		resultTag = "data";

	if(resultTag.compare(name) == 0){
	    for(int i = 0; ; i = i + 2){
	    	if(atts[i] == NULL)	break;

	    	if(strcmp(atts[i], "score") == 0){
	    		// スコア
	    		scoreStr = std::string(atts[i + 1]);
	    	}
	    	if(strcmp(atts[i], "text") == 0){
	    		// 候補名
	    		candidateName = std::string(atts[i + 1]);
	    	}
	    }
    	result = pMain->AddCandidate(candidateName, atof(scoreStr.c_str()));
	}
}

void element_end(void *userData, const XML_Char *name)
{
#ifdef _DEBUG2
	printf("[ELEMENT] %s End!\n", name);
#endif
}

// Module specification
// <rtc-template block="module_spec">
static const char* terminalserver_spec[] =
  {
    "implementation_id", "TerminalServer",
    "type_name",         "TerminalServer",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "RTC-Center",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.HttpServerAddress", "192.168.30.155",
    "conf.default.JuliusScoreLimit", "0.5",
//    "conf.default.CandidateIndex", "-1",
    "conf.default.RtcCentMemberFile", "MemberData",
    "conf.default.CommandScore", "0.7",
    "conf.default.AnnounceWait", "0",
    // Widget
    "conf.__widget__.conf_name0", "text",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
TerminalServer::TerminalServer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_rtc_cgi_requestIn("CgiRequest", m_rtc_cgi_request),
    m_recv_gtalk_msgIn("GTalkRecvMsg", m_recv_gtalk_msg),
    m_recv_gtalk_statusIn("GTalkRecvStatus", m_recv_gtalk_status),
#ifdef __RS001
    m_recv_juliusInside_statIn("JuliusInsideStatus", m_recv_juliusInside_stat),
    m_recv_juliusInsideIn("JuliusInsideResult", m_recv_juliusInside),
    m_recv_juliusOutside_statIn("JuliusOutsideStatus", m_recv_juliusOutside_stat),
    m_recv_juliusOutsideIn("JuliusOutsideResult", m_recv_juliusOutside),
    m_recv_juliusOrg_statIn("JuliusOrgStatus", m_recv_juliusOrg_stat),
    m_recv_juliusOrgIn("JuliusOrgResult", m_recv_juliusOrg),
    m_recv_juliusNum_statIn("JuliusNumStatus", m_recv_juliusNum_stat),
    m_recv_juliusNumIn("JuliusNumResult", m_recv_juliusNum),
#endif
#ifdef __RS001_1
    m_recv_julius_statIn("JuliusStatus", m_recv_julius_stat),
    m_recv_juliusIn("JuliusResult", m_recv_julius),
#endif
	m_send_juliusDataIn("JuliusDataIn", m_send_juliusData),
	m_jtalk_statusIn("JTalkStatusIn", m_jtalk_status),
	m_MonitorDataIn("MonitorDataIn", m_MonitorData),
	m_send_juliusDataOut("JuliusDataOut", m_send_juliusData),
    m_cycle_dataOut("CycleData", m_cycle_data),
    m_send_gtalk_msgOut("GTalkSend", m_send_gtalk_msg),
    m_send_voiceOut("Announce", m_send_voice),
#ifdef __RS001_1
	m_julius_grammar_typeOut("JuliusGrammarType", m_julius_grammarData),
#endif
    m_PA10CtrlPort(/*"PA10Ctrl"*/"PA10SystemControl"),
    m_RHCtrlPort("RefHardCtrl"),
	m_StateChangePA10Port("PA10StateChage"),
	m_StateChangeRHPort("RHStateChage")
    // </rtc-template>
{
#ifdef SCENARIO_FILE
		// シナリオコマンドリスト生成
		createScenarioCommandList();
#endif
}

/*!
 * @brief destructor
 */
TerminalServer::~TerminalServer()
{
}



RTC::ReturnCode_t TerminalServer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  addInPort("CgiRequest", m_rtc_cgi_requestIn);

  addInPort("GTalkRecvMsg", m_recv_gtalk_msgIn);
  addInPort("GTalkRecvStatus", m_recv_gtalk_statusIn);
#ifdef __RS001
  addInPort("JuliusInsideStatus", m_recv_juliusInside_statIn);
  addInPort("JuliusInsideResult", m_recv_juliusInsideIn);
  addInPort("JuliusOutsideStatus", m_recv_juliusOutside_statIn);
  addInPort("JuliusOutsideResult", m_recv_juliusOutsideIn);
  addInPort("JuliusOrgStatus", m_recv_juliusOrg_statIn);
  addInPort("JuliusOrgResult", m_recv_juliusOrgIn);
  addInPort("JuliusNumStatus", m_recv_juliusNum_statIn);
  addInPort("JuliusNumResult", m_recv_juliusNumIn);
#endif

#ifdef __RS001_1
	addInPort("JuliusStatus", m_recv_julius_statIn);
	addInPort("JuliusResult", m_recv_juliusIn);
#endif

  addInPort("JuliusDataIn", m_send_juliusDataIn);

  addInPort("JTalkStatusIn", m_jtalk_statusIn);
  addInPort("MonitorDataIn", m_MonitorDataIn);
  // Set OutPort buffer
  addOutPort("CycleData", m_cycle_dataOut);

  addOutPort("GTalkSend", m_send_gtalk_msgOut);
  addOutPort("Announce", m_send_voiceOut);

  addOutPort("JuliusDataOut", m_send_juliusDataOut);
  // Set service provider to Ports

#ifdef __RS001_1
	addOutPort("JuliusGrammarType", m_julius_grammar_typeOut);
#endif
  // Set service consumers to Ports
  m_PA10CtrlPort.registerConsumer("SystemControl", "PA10SystemControl", m_pa10if);
//  m_PA10CtrlPort.registerConsumer("PA10Ctrl", "PA10Ctrl", m_pa10if);
  m_RHCtrlPort.registerConsumer("RefHardCtrl", "RefHardCtrl", m_rh_if);

	m_StateChangePA10Port.registerProvider("StateChangeFromPA10", "StateChangeFromPA10" , m_StateChangePA10);
	m_StateChangeRHPort.registerProvider("StateChangeFromRH", "StateChangeFromRH" , m_StateChangeRH);

  // Set CORBA Service Ports
  addPort(m_PA10CtrlPort);
  addPort(m_RHCtrlPort);
  addPort(m_StateChangePA10Port);
  addPort(m_StateChangeRHPort);

	m_StateChangePA10.SetParent(this);
	m_StateChangeRH.SetParent(this);

  // </rtc-template>
  bindParameter("HttpServerAddress", m_HttpServerIP, "192.168.30.155");
  bindParameter("JuliusScoreLimit", m_JuliusScore, "0.5");
//  bindParameter("CandidateIndex", m_CandidateIndex, "-1"); m_RtcCentMemberFile
  bindParameter("RtcCentMemberFile", m_RtcCentMemberFile, "MemberData");
  bindParameter("CommandScore", m_CommandScore, "0.7");
  bindParameter("AnnounceWait", m_AnnounceWait, "0");

  InitializeFlags();
	m_parser = XML_ParserCreate(NULL);
	m_TickerNum			= 1;
	m_SystemEnd			= false;
	m_SystemState		= 0;
	m_ScenarioStatus	= 0;

#ifdef PA10_RH_CTRL
	// ワーカースレッド起動
	{
		int	sysRet;

		sysRet = pthread_create(&m_thread_id, 0, sceralio_running, this);
	}
#endif

#ifdef _DEBUG
  printf("OnInitialize\n");
#endif
#ifdef _ALIVE_RTC
	aliveLibInit(m_pManager, this);
	systemStateData.createData();
#endif

  return RTC::RTC_OK;
}


void TerminalServer::InitializeFlags()
{
	m_Visitor					= false;
	m_JuliusInsideLastState		= "";
	m_JuliusInsideLastMsg		= "";
	m_JuliusOutsideLastState	= "";
	m_JuliusOutsideLastMsg		= "";
	m_JuliusOrgLastState		= "";
	m_JuliusOrgLastMsg			= "";
	m_JuliusNumLastState		= "";
	m_JuliusNumLastMsg			= "";
	m_CandidateStage			= 0;
	m_UserOperation				= false;
	m_Announce					= 0;
	JuliusResult				= "";
	JTalkStatus					= "";
	JuliusStatus				= "";
}

RTC::ReturnCode_t TerminalServer::onFinalize()
{
	int	sysRet;

#ifdef _DEBUG
	printf("onFinalize\n");
#endif

	if(m_parser != NULL){
		XML_ParserFree(m_parser);
	}
	m_SystemEnd = true;
	sysRet = pthread_join(m_thread_id, 0);

	return RTC::RTC_OK;
}



RTC::ReturnCode_t TerminalServer::onStartup(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
printf("OnStartup\n");
#endif

  return RTC::RTC_OK;
}



RTC::ReturnCode_t TerminalServer::onShutdown(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	printf("onShutdown\n");
#endif

  return RTC::RTC_OK;
}



RTC::ReturnCode_t TerminalServer::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	printf("onActivated\n");
#endif
#ifdef _ALIVE_RTC
	aliveLibActivated();
#endif
	m_MemberInfo.readData(m_RtcCentMemberFile);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t TerminalServer::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	printf("onDeactivated\n");
#endif

  return RTC::RTC_OK;
}



RTC::ReturnCode_t TerminalServer::onExecute(RTC::UniqueId ec_id)
{
	std::string StatusStr;
	int			VoiceRecognition = 0;

#ifdef _ALIVE_RTC
	aliveLibIsAlive();
#endif
	// 発行済み来訪者チケットの状態確認
	CheckTicket();

	if(m_MonitorDataIn.isNew()){
		m_MonitorDataIn.read();
		std::cout << "MonitorData = " << m_MonitorData.data << std::endl;
	}
	// Juliusにデータを流す
	if(m_send_juliusDataIn.isNew()){
		m_send_juliusDataIn.read();
		if(m_CandidateStage > 0)
			m_send_juliusDataOut.write();
	}
#ifdef __RS001
	// JuliusRTCステータス変更処理
	if(m_recv_juliusInside_statIn.isNew()){
		m_recv_juliusInside_statIn.read();
#ifdef _DEBUG
		std::cout << "JuliusInsideStatus = " << m_recv_juliusInside_stat.data << std::endl;
#endif
	}
	// JuliusRTC結果取得処理
	if(m_recv_juliusInsideIn.isNew()){
		m_recv_juliusInsideIn.read();
#ifdef _DEBUG
		std::cout << "JuliusInsideResult = " << m_recv_juliusInside.data << std::endl;
#endif
	}

	// JuliusRTCステータス変更処理
	if(m_recv_juliusOutside_statIn.isNew()){
		m_recv_juliusOutside_statIn.read();
#ifdef _DEBUG
		std::cout << "JuliusOutsideStatus = " << m_recv_juliusOutside_stat.data << std::endl;
#endif
	}
	// JuliusRTC結果取得処理
	if(m_recv_juliusOutsideIn.isNew()){
		m_recv_juliusOutsideIn.read();
#ifdef _DEBUG
		std::cout << "JuliusOutsideResult = " << m_recv_juliusOutside.data << std::endl;
#endif
	}

	// JuliusRTCステータス変更処理
	if(m_recv_juliusOrg_statIn.isNew()){
		m_recv_juliusOrg_statIn.read();
#ifdef _DEBUG
		std::cout << "JuliusOrgStatus = " << m_recv_juliusOrg_stat.data << std::endl;
#endif
	}
	// JuliusRTC結果取得処理
	if(m_recv_juliusOrgIn.isNew()){
		m_recv_juliusOrgIn.read();
#ifdef _DEBUG
		std::cout << "JuliusOrgResult = " << m_recv_juliusOrg.data << std::endl;
#endif
	}

	// JuliusRTCステータス変更処理
	if(m_recv_juliusNum_statIn.isNew()){
		m_recv_juliusNum_statIn.read();
#ifdef _DEBUG
		std::cout << "JuliusNumStatus = " << m_recv_juliusNum_stat.data << std::endl;
#endif
	}
	// JuliusRTC結果取得処理
	if(m_recv_juliusNumIn.isNew()){
		m_recv_juliusNumIn.read();
#ifdef _DEBUG
		std::cout << "JuliusNumResult = " << m_recv_juliusNum.data << std::endl;
#endif
	}
#endif
#ifdef __RS001_1

	// JuliusRTCステータス変更処理
	if(m_recv_julius_statIn.isNew()){
		m_recv_julius_statIn.read();
#ifdef _DEBUG
		std::cout << "JuliusStatus = " << m_recv_julius_stat.data << std::endl;
#endif
		JuliusStatus = m_recv_julius_stat.data;
		if(JuliusStatus.compare("LISTEN") == 0){
			// 音声入力待機中
		}
		if(JuliusStatus.compare("STARTREC") == 0){
			// 音声入力中
		}
		if(JuliusStatus.compare("ENDREC") == 0){
			// 音声入力終了
		}
	}
	// JuliusRTC結果取得処理
	if(m_recv_juliusIn.isNew()){
		m_recv_juliusIn.read();
#ifdef _DEBUG
		std::cout << "JuliusResult = " << m_recv_julius.data << std::endl;
#endif
		JuliusResult = m_recv_julius.data;
		if(JuliusResult.length() > 0){
			// 音声解析終了、結果出力
		}
	}

#endif
	if (m_parser != NULL) {
		std::string	xmlStr;
#ifdef __RS001
		switch(m_CandidateStage){
		case 1:
			xmlStr = m_recv_juliusOrg.data;
			break;
		case 2:
			xmlStr = m_recv_juliusOutside.data;
			break;
		case 3:
			xmlStr = m_recv_juliusInside.data;
			break;
		case 4:
			xmlStr = m_recv_juliusNum.data;
			break;
		}
#endif
#ifdef __RS001_1
		xmlStr = m_recv_julius.data;
#endif
		if(xmlStr.length() > 0){
			XML_ParserReset(m_parser, NULL);
			XML_SetUserData(m_parser, this);
			XML_SetElementHandler(m_parser, parse_julius_result, element_end);
			m_CandidateList.clear();
			if ((XML_Parse(m_parser, xmlStr.c_str(), xmlStr.length(), 1)) == 0) {
#ifndef _NOT_ERR_MSG
				std::cout << "XML Parse Error!" << std::endl;
#endif
			}
		}
	}

	// GTalkメッセージ処理
	if(m_recv_gtalk_msgIn.isNew()){
		m_recv_gtalk_msgIn.read();
#ifdef _DEBUG
		std::cout << "Gtalk Message = " << m_recv_gtalk_msg.data;
		std::cout << "\n";
#endif
	}

	// GTalkステータス変更処理
	if(m_recv_gtalk_statusIn.isNew()){
		GtalkStatusChange();
	}

	// CGIリクエスト処理
	if(m_rtc_cgi_requestIn.isNew()){
		CGI_Proc();
	}

	if(m_jtalk_statusIn.isNew()){
		m_jtalk_statusIn.read();
#ifdef _DEBUG
		std::cout << "JTalkStatus = " << m_jtalk_status.data;
		std::cout << "\n";
#endif
		JTalkStatus = m_jtalk_status.data;

		if(JTalkStatus.compare("started") == 0){
			// 発話開始
			// 発話中は音声認識が作動しない・・・
			m_Announce = 0;
		}

		if(JTalkStatus.compare("finished") == 0){
			// 終話数秒前(5秒前くらい)
			// 終了した！ じゃなくて終了するっぽいなので、ちょっと待機する必要がある
			m_Announce = time(NULL) + m_AnnounceWait;
			VoiceRecognition = 1;
		}
	}

	// 音声認識可/不可確認
	{
		/*
		int				VoiceRecognition = 0;
		std::string		JuliusResult;
		std::string		JTalkStatus;
		std::string		JuliusStatus;
		*/

		if(m_Announce > 0)	VoiceRecognition = 1;
		if((m_Announce != 0) && (m_Announce < time(NULL))){
			if(JuliusStatus.compare("ENDREC") == 0)			VoiceRecognition = 3;
			if(JuliusStatus.compare("STARTREC") == 0)		VoiceRecognition = 2;
			if(JuliusStatus.compare("LISTEN") == 0)			VoiceRecognition = 5;
			JTalkStatus = "";
		}
		if((JTalkStatus.compare("started") == 0) || (JTalkStatus.compare("finished") == 0))
			VoiceRecognition = 1;
	}

	// 周期メッセージ構築処理
	/*
	 * メッセージの形式は以下の形式と仮定する。
	 * MsgData=メッセージデータ/hh:mm:ss(メッセージ登録時刻)/宛先,メッセージデータ/hh:mm:ss(メッセージ登録時刻)/宛先.・・・
	 * RobotStatus=PA10/Waiting,RH/Running,・・・・
	 * SystemStatus=Normal/Waiting
	 * RHPos=10.5/30.9/70.0(X/Y/ω)
	 * Visitor=none
	 * Users=yotubata,tnninomi,・・・
	 *
	 * 各項目は「;(セミコロン)」で結合する。
	 */

	StatusStr = "";
#if 0
//	StatusStr = "MsgData=メッセージ&データ/12:00:01/yotubata/all/1,メッセージデータ2/11:09:00/tnninomi/yotubata/2;";
	StatusStr = StatusStr + "RobotStatus=PA10/Waiting,RH/Running;";
	StatusStr = StatusStr + "SystemStatus=Normal/Waiting;";
	StatusStr = StatusStr + "RHPos=10.5/30.9/70.0;";
	if(m_Visitor)
		StatusStr = StatusStr + "Visitor=True;";
	else
		StatusStr = StatusStr + "Visitor=none;";
	switch(m_Answer){
		case 1:
			StatusStr = StatusStr + "Answer=1;";
			break;
		case 2:
			StatusStr = StatusStr + "Answer=2;";
			break;
		case 3:
			StatusStr = StatusStr + "Answer=3;";
			break;
	}
//	StatusStr = StatusStr + "Users=yotubata,tnninomi;";
#else
	/*
	 データの形式はXMLとし、以下のようなXMLを出力する
		<?xml version="1.0" encoding="utf-8"?>
		<SystemStatus target="all" value="Normal" />
		<RHInfo target="Terminal" X="0.0" Y="0.0" O="90" status="Normal" />
		<PA10Info target="Terminal" status="Normal" />
		<FrontDesk target="192.168.30.X">
			<Answer value="X" />
			<Camera value="True" />
			<CandidateList type="InsideName">
				<Candidate text="津幡" />
				<Candidate text="伊藤" />
				<Candidate text="高橋" />
				<Candidate text="鈴木" />
				<Candidate text="小笠原" />
			<CandidateList>
			<CandidateList type="OutsideName">
				<Candidate text="津幡" />
				<Candidate text="伊藤" />
				<Candidate text="高橋" />
				<Candidate text="鈴木" />
				<Candidate text="小笠原" />
			<CandidateList>
			<CandidateList type="OrgName">
				<Candidate text="富士ソフト" />
				<Candidate text="産業技術総合研究所" />
				<Candidate text="東京大学" />
				<Candidate text="千葉工業大学" />
				<Candidate text="日本ロボット工業会" />
			<CandidateList>
		</FrontDesk>
		<MeetingAns target="192.168.30.XXX">
		</MeetingAns>
	 */
#endif
	//m_cycle_data.data= StatusStr;
#ifndef _TEST_DATA
	StatusStr = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	StatusStr = StatusStr + "<IntegratedVerification> \n";
	StatusStr = StatusStr + "\t<SystemStatus target=\"all\" value=\"Normal\" status=\"Normal\" service=\"Wait\" /> \n";
	StatusStr = StatusStr + "\t<ScenarioStatus target=\"all\" value=\"" + int_to_string(m_ScenarioStatus) + "\" />\n";
	StatusStr = StatusStr + "\t<RHInfo target=\"Terminal\" X=\"0.0\" Y=\"0.0\" O=\"90\" status=\"Normal\" service=\"Wait\" /> \n";
	StatusStr = StatusStr + "\t<PA10Info target=\"Terminal\" status=\"Normal\" service=\"Wait\" /> \n";
	StatusStr = StatusStr + "\t<FrontDesk target=\"192.168.30.X\" status=\"Normal\" service=\"Wait\" >\n";

	if(m_Visitor)
		StatusStr = StatusStr + "\t\t<Camera value=\"True\" />\n";
	else
		StatusStr = StatusStr + "\t\t<Camera value=\"none\" />\n";

	switch(m_Answer){
		case 1:
			StatusStr = StatusStr + "\t\t<Answer value=\"1\" />\n";
			break;
		case 2:
			StatusStr = StatusStr + "\t\t<Answer value=\"2\" />\n";
			break;
		case 3:
			StatusStr = StatusStr + "\t\t<Answer value=\"3\" />\n";
			break;
	}

	if(m_CandidateList.size() > 0){
		if(m_CandidateStage == 3){
			int cnt = 1;
			std::stringstream ss;

			StatusStr = StatusStr + "\t\t<CandidateList type=\"InsideName\">\n";
			for(VECTOR_ITERATOR it = m_CandidateList.begin(); it != m_CandidateList.end(); it++, cnt++){
				ss << cnt;
				StatusStr = StatusStr + "\t\t\t<Candidate text=\"" + ss.str() + "." + *it + "\" />\n";
			}
			StatusStr = StatusStr + "\t\t</CandidateList>\n";
		}

		if(m_CandidateStage == 2){
			int cnt = 1;
			std::stringstream ss;

			StatusStr = StatusStr + "\t\t<CandidateList type=\"OutsideName\">\n";
			for(VECTOR_ITERATOR it = m_CandidateList.begin(); it != m_CandidateList.end(); it++, cnt++){
				ss << cnt;
				StatusStr = StatusStr + "\t\t\t<Candidate text=\"" + ss.str() + "." + *it + "\" />\n";
			}
			StatusStr = StatusStr + "\t\t</CandidateList>\n";
		}

		if(m_CandidateStage == 1){
			int cnt = 1;
			std::stringstream ss;

			StatusStr = StatusStr + "\t\t<CandidateList type=\"OrgName\">\n";
			for(VECTOR_ITERATOR it = m_CandidateList.begin(); it != m_CandidateList.end(); it++, cnt++){
				ss << cnt;
				StatusStr = StatusStr + "\t\t\t<Candidate text=\"" + ss.str() + "." + *it + "\" />\n";
			}
			StatusStr = StatusStr + "\t\t</CandidateList>\n";
		}
	}
	if(m_CandidateStage == 4){
		if(m_CandidateList.size() > 0){
			StatusStr = StatusStr + "\t\t<CandidateSelect value=\"" + m_CandidateList[0] + "\" />\n";
		}
		else{
			StatusStr = StatusStr + "\t\t<CandidateSelect value=\"-1" + "\" />\n";
		}
	}

	if(m_CandidateStage == 9){
		if(m_CandidateList.size() > 0){
			StatusStr = StatusStr + "\t\t<ShortCommand value=\"" + m_CandidateList[0] + "\" />\n";
		}
		else{
			StatusStr = StatusStr + "\t\t<ShortCommand value=\"" + "\" />\n";
		}
	}

	if(m_CandidateStage == 0){
		StatusStr = StatusStr + "\t\t<CandidateSelect value=\"-1" + "\" />\n";
	}

	{
		std::stringstream ss;

		ss << m_CandidateStage;
		StatusStr = StatusStr + "\t\t<CandidateStage value=\"" + ss.str() + "\" />\n";
	}

	{
		std::stringstream ss;

		ss << VoiceRecognition;
		StatusStr = StatusStr + "\t\t<VoiceRecognitionState value=\"" + ss.str() + "\" />\n";
	}

	StatusStr = StatusStr + "\t</FrontDesk>\n";

	StatusStr = StatusStr + "\t<Meeting target=\"192.168.30.XXX\" status=\"Normal\" service=\"Wait\" > \n";
	if(m_Incoming2)
		StatusStr = StatusStr + "\t\t<RHMoveFinished value=\"True\" />\n";
	else
		StatusStr = StatusStr + "\t\t<RHMoveFinished value=\"False\" />\n";
	StatusStr = StatusStr + "\t</Meeting> \n";
	StatusStr = StatusStr + "</IntegratedVerification>\n";
#else
	StatusStr = "<?xml version=\"1.0\" encoding=\"utf-8\"?> \
		<IntegratedVerification> \
		<SystemStatus target=\"all\" value=\"Normal\" /> \
		<RHInfo target=\"Terminal\" X=\"0.0\" Y=\"0.0\" O=\"90\" status=\"Normal\" /> \
		<PA10Info target=\"Terminal\" status=\"Normal\" />";

	StatusStr = StatusStr + " <FrontDesk target=\"192.168.30.X\">";
	StatusStr = StatusStr + "<Camera value=\"True\" />";
	StatusStr = StatusStr + "<Answer value=\"1\" />";

	StatusStr = StatusStr + "<CandidateList type=\"InsideName\">";
	StatusStr = StatusStr + "<Candidate text=\"1.内部1\" />";
	StatusStr = StatusStr + "<Candidate text=\"2.内部2\" />";
	StatusStr = StatusStr + "<Candidate text=\"3.内部3\" />";
	StatusStr = StatusStr + "</CandidateList>";

	StatusStr = StatusStr + "<CandidateList type=\"OutsideName\">";
	StatusStr = StatusStr + "<Candidate text=\"1.外部1\" />";
	StatusStr = StatusStr + "<Candidate text=\"2.外部2\" />";
	StatusStr = StatusStr + "<Candidate text=\"3.外部3\" />";
	StatusStr = StatusStr + "</CandidateList>";

	StatusStr = StatusStr + "<CandidateList type=\"OrgName\">";
	StatusStr = StatusStr + "<Candidate text=\"1.企業1\" />";
	StatusStr = StatusStr + "<Candidate text=\"2.企業2\" />";
	StatusStr = StatusStr + "<Candidate text=\"3.企業3\" />";
	StatusStr = StatusStr + "</CandidateList>";
//	StatusStr = StatusStr + "<CandidateSelect value=\"" + m_CandidateIndex + "\" />";

	StatusStr = StatusStr + "</FrontDesk>";
	StatusStr = StatusStr + " </IntegratedVerification>";

#endif
	m_cycle_data.data= (const char*)(StatusStr.c_str());
//	m_cycle_dataOut.length(StatusStr.length());
	m_cycle_dataOut.write();
#if 0
	std::cout << "\n\n===============" << std::endl;
	std::cout << StatusStr << std::endl;
#endif
#ifdef __RS001
	m_recv_juliusOrg.data		= "";
	m_recv_juliusOutside.data	= "";
	m_recv_juliusInside.data	= "";
	m_recv_juliusNum.data		= "";
#endif
#ifdef __RS001_1
	m_julius_grammarData.data	= "";
	m_recv_julius_stat.data		= "";
	m_recv_julius.data			= "";
#endif

	return RTC::RTC_OK;
}
void TerminalServer::GtalkStatusChange()
{
	std::string	xmlStr;

	m_recv_gtalk_statusIn.read();
	/*
	 * メンバーステータス解析
	 * ステータス情報はXML形式でくるので（以下を参照）XMLをパースしてステータスを取り出す必要がある。
	 * ステータスの取得先は現時点で３種類
	 * 	・gmail
	 * 	・iGoogle
	 * 	・Talk
	 * ステータスは2種類
	 * 	・available		（在籍）
	 * 	・unavailable	（不在）
		<?xml version="1.0" encoding="utf-8"?><status from="yotubata@fsi.co.jp/gmail.8D74DE2E" state="available"/>
		<?xml version="1.0" encoding="utf-8"?><status from="yotubata@fsi.co.jp/iGoogle7A43BE53" state="away"/>
		<?xml version="1.0" encoding="utf-8"?><status from="yotubata@fsi.co.jp/Talk.v105F58C19DD" state="available"/>
		<?xml version="1.0" encoding="utf-8"?><status from="yotubata@fsi.co.jp/Talk.v105F58C19DD" state="unavailable"/>
	*/
#ifdef _DEBUG
	std::cout << "GTalk Status = " << m_recv_gtalk_status.data;
	std::cout << "\n";
#endif
	xmlStr = m_recv_gtalk_status.data;
	if (m_parser != NULL) {
		XML_ParserReset(m_parser, NULL);
		XML_SetUserData(m_parser, this);
		XML_SetElementHandler(m_parser, parse_gtalk_state, element_end);
		if ((XML_Parse(m_parser, xmlStr.c_str(), xmlStr.length(), 1)) == 0) {
#ifndef _NOT_ERR_MSG
			std::cout << "XML Parse Error!" << std::endl;
#endif
		}
	}
}

void TerminalServer::CheckTicket()
{
	// チケットが登録されていたら、時刻の確認を行う
	for(TICKET_IT it = m_TicketList.begin(); it != m_TicketList.end(); it++){
		if(it->second.m_Closed == true){
			// チケット削除
			m_TicketList.erase(it);
			break;
		}
		else{
			// 時刻確認
			time_t pCurTime, curTime;

			curTime = time(&pCurTime);
			if(curTime >= (it->second.m_VisitTime + 30)){
				std::string StatusStr;
				std::string	Responder;
				std::string	msgBody;
				std::string PIC = it->second.m_PersonInCharge;
				std::vector<std::string>	toMember;

				toMember = m_MemberInfo.GetAvailableList();

				if(PIC.length() == 0) PIC = "受付担当者";
				// メッセージ送信
				msgBody = msgBody + "\" body=\"" + PIC;
				msgBody = msgBody + "宛にお客様がお見えになりましたが、不在です。\n";
				msgBody = msgBody + "来訪者情報確認サイトにアクセスして対応方法の入力をお願いします。\n";
				msgBody = msgBody + "来訪者情報確認URL : http://" + m_HttpServerIP + ":6809/cgi-bin/terminalServer.cgi?";
				msgBody = msgBody + "Function=FrontDesk";
				msgBody = msgBody + "&VisitorNumber=" + it->second.m_VisitNumber;

				for(std::vector<std::string>::iterator send_it = toMember.begin(); send_it != toMember.end(); send_it++){
					Responder = "&Responder=" + m_MemberInfo.GetNameFromMailAddr(*send_it);

					StatusStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
					StatusStr = StatusStr + "<message to=\"";
					StatusStr = StatusStr + *send_it;
					StatusStr = StatusStr + msgBody;
					StatusStr = StatusStr + Responder;
					StatusStr = StatusStr + "\" />";
					// GTalkにメッセージを送信する
					if(StatusStr.length() > 0){
						m_send_gtalk_msg.data = StatusStr.c_str();
						m_send_gtalk_msgOut.write();
						std::cout << StatusStr << std::endl;
					}
				}

				// 訪問時刻を変更して、再度３０秒待ちとする
				curTime = time(&pCurTime);
				it->second.m_VisitTime = curTime;
			}
		}
	}
}

void TerminalServer::CGI_Proc()
{
	QueryInfo	requestInfo;
	std::string	cgi_str;

	m_rtc_cgi_requestIn.read();
	/*
	 * m_rtc_cgi_request.data は以下の形式となっている
	 * XXX=YYYY&ZZZZ=AAAA
	 * すなわち、http://aaaa.cgi?XXX=YYYY&ZZZZ=AAAA の
	 * ?以降の文字列が格納されてくる。
	 */
#ifdef _DEBUG
	std::cout << m_rtc_cgi_request.data;
	std::cout << "\n\n\n";
#endif
	cgi_str = m_rtc_cgi_request.data;
	requestInfo.ParseQuery(cgi_str);

	// 来訪者検出状態参照
	if(requestInfo.GetQueryPair("Visitor") == "True"){
		if(m_SystemState == 0){
			m_Visitor = true;
			m_SystemState = 1;
		}
	}
	// 来訪者がタッチパネルに触った
	if(requestInfo.GetQueryPair("Visitor") == "Fix"){
		m_Visitor = false;
	}
	// 来訪者情報取得
	if(requestInfo.GetQueryPair("VisitorData") == "True"){
		std::cout << "VisitorData IN" << std::endl;
		VisitorProc(&requestInfo);
	}
	// 来訪者取扱い結果処理
	if(requestInfo.GetQueryPair("FrontDeskAnswer") == "True"){
		m_Answer = atoi(requestInfo.GetQueryPair("AnswerVal").c_str());
		m_TicketList[requestInfo.GetQueryPair("TicketNum")].m_Closed = true;
	}
	// 端末状態通知処理
	if(requestInfo.GetQueryPair("Command") == "SetStatus"){
		TerminalStatus	terminalInfo;

		m_SystemState = 0;
		m_Visitor = false;

		if(m_TerminalList.find(requestInfo.GetQueryPair("MchID")) != m_TerminalList.end()){
			// すでにデータがある
			terminalInfo = m_TerminalList[requestInfo.GetQueryPair("MchID")];
			terminalInfo.status		= requestInfo.GetQueryPair("Status");
			m_TerminalList[requestInfo.GetQueryPair("MchID")] = terminalInfo;
		}
		else{
			// 新規追加
			std::string	ip_addr;
			terminalInfo.ip_addr	= requestInfo.GetQueryPair("MchID");
			terminalInfo.attribute	= requestInfo.GetQueryPair("Attribute");
			terminalInfo.status		= requestInfo.GetQueryPair("Status");
			terminalInfo.camera_ip	= requestInfo.GetQueryPair("BuddyCamera");
//			m_TerminalList[terminalInfo.ip_addr] = terminalInfo;
			m_TerminalList.insert(TERMINAL_INFO_VAL(terminalInfo.ip_addr, terminalInfo));
		}
#if 1	// 暫定処理
		// 状態フラグをすべて初期化する
		if(requestInfo.GetQueryPair("Status") == "Start"){
			InitializeFlags();
		}
#endif
	}

	// シナリオ実行状態変更要求
	if(requestInfo.GetQueryPair("Command") == "ScenarioControl"){
		m_ScenarioStatus = string_to_int(requestInfo.GetQueryPair("Value"));
		if(m_ScenarioStatus == 0){
			m_scenarioList.clear();
			m_Incoming		= 0;				// RH到着シグナル
			m_Incoming2		= 0;				// RH到着シグナル
			m_PA10Waiting	= 0;				// PA10ドリンク積載終了
			std::cout << "シナリオ強制終了" << std::endl;
		}
	}

	// 端末音声要求
	if(requestInfo.GetQueryPair("Command") == "PlayVoice"){
		std::string	VoiceText = requestInfo.GetQueryPair("Voice");

		if(VoiceText.length() > 0){
			m_send_voice.data = VoiceText.c_str();
			m_send_voiceOut.write();
		}
	}

	// 音声認識要求
	if(requestInfo.GetQueryPair("Command") == "VoiceRecognition"){
		std::string	StageText = requestInfo.GetQueryPair("Stage");

//		m_CandidateIndex = "-1";
#ifdef __RS001
		m_recv_juliusOrg.data		= "";
		m_recv_juliusOutside.data	= "";
		m_recv_juliusInside.data	= "";
		m_recv_juliusNum.data		= "";
#endif
#ifdef __RS001_1
		m_julius_grammarData.data	= "";
		m_recv_julius_stat.data		= "";
		m_recv_julius.data			= "";
#endif
		m_CandidateList.clear();
		if(StageText.length() > 0){
			std::string		grammarName;
			m_CandidateStage = atoi(StageText.c_str());
#ifdef __RS001_1
			switch(m_CandidateStage){
			case 1:	// 訪問者会社名
				grammarName = "irp-associates";
				break;
			case 2:	// 訪問者名
				grammarName = "names-tiny";
				break;
			case 3:	// 訪問先担当者名
				grammarName = "rtc-center-names";
				break;
			case 4:	// 番号
				grammarName = "number";
				break;
			case 9:
				grammarName = "word";
				break;
			default:	// その他
				grammarName = "";
				break;
			}
			if(grammarName.length() > 0){
				// 辞書切り替え
				m_julius_grammarData.data = grammarName.c_str();
				m_julius_grammar_typeOut.write();
			}
#endif
		}
	}
#ifdef SCENARIO_FILE
	// サービス要求
	if(requestInfo.GetQueryPair("Command") == "ServiceReq"){
		SCEANRIO_DYNA_PARAM_MAP	paramList;
		std::string				paramName;

		m_Incoming		= 0;				// RH到着シグナル
		m_Incoming2		= 0;				// RH到着シグナル
		m_PA10Waiting	= 0;				// PA10ドリンク積載終了

		for(int paramCnt = 1; ; paramCnt++){
			paramName = "Param" + int_to_string(paramCnt);
			if(requestInfo.GetQueryPair(paramName).length() > 0){
				paramList.insert(SCEANRIO_DYNA_PARAM_VAL(paramName, requestInfo.GetQueryPair(paramName)));
			}
			else{
				break;
			}
		}
		readScenarioFile(requestInfo.GetQueryPair("ScenarioFile"), paramList);
		m_ScenarioStatus = 1;
	}
#else
	// 給仕要求
	if(requestInfo.GetQueryPair("Command") == "WaitService"){
		m_DrinkNum = atoi(requestInfo.GetQueryPair("Count").c_str());
		createScenario(4);
	}
	// 廃棄要求
	if(requestInfo.GetQueryPair("Command") == "DisposalService"){
		createScenario(8);
	}
#endif
	// ユーザー操作完了通知
	if(requestInfo.GetQueryPair("Command") == "UserOperate"){
		m_UserOperation = true;
	}
}

#ifdef PA10_RH_CTRL
#ifdef SCENARIO_FILE
void createScenarioCommandList()
{
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("案内",				11 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("給仕開始",			21 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("給仕搬送",			22 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("廃棄開始",			31 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("廃棄搬送",			32 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("PA10積載",			41 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("RH帰還",			51 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("呼出到着待機",		91 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("PA10積載終了待機",	92 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("受取待機",			93 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("空き缶積載終了待機",	94 ));
	ScenarioCommand.insert(SCEANRIO_CMD_VAL("RH到着待機",		95 ));
}

void TerminalServer::readScenarioFile(std::string fileName, SCEANRIO_DYNA_PARAM_MAP paramList)
{
	// ファイルを読み込んで、シナリオリストを生成する
	if(access(fileName.c_str(), F_OK) == 0){
		// ファイルを開いてデータを読み込む
		// 氏名、仮名、名字、仮名、メールアドレス、受付担当フラグ　1行1レコード
		char		lineData[1024];
		int			line = 0;
		std::string	data;
		std::string::size_type index;

		// ファイルストリームを作成する
		std::ifstream ifs(fileName.c_str(), std::ios::in);
		if(ifs){
			ScenarioInfo	scenalio_info;

			for(line = 0; !ifs.eof(); line++){

				memset(lineData, 0, sizeof(lineData));
				ifs.getline(lineData, sizeof(lineData));
				data = lineData;
				if(data.length() > 1){
					// コメント行は削除する
					index = data.find(";");
					if(index != std::string::npos){
						data = data.substr(0, index);
					}
					index = data.find("#");
					if(index != std::string::npos){
						data = data.substr(0, index);
					}
					// データありなのでシナリオを読み込む
					if(data.length() > 1){
						std::vector<std::string>	recordData;
						// 給仕開始要求		( 21 )
						recordData = string_token(data, " ,");

						scenalio_info.commandID = ScenarioCommand[recordData[0].c_str()];
						scenalio_info.param.clear();
						for(unsigned int paramCnt = 1; paramCnt < recordData.size(); paramCnt++){
							if(paramList.find(recordData[paramCnt]) != paramList.end()){
								// パラメタ展開
								scenalio_info.param.push_back(paramList[recordData[paramCnt]]);
							}
							else{
								scenalio_info.param.push_back(recordData[paramCnt]);
							}
						}
						m_scenarioList.push_back(scenalio_info);
					}
				}
			}
			// シナリオ終了命令を付加する
			scenalio_info.commandID = 999;
			m_scenarioList.push_back(scenalio_info);
		}
	}
	else{
		std::cout << fileName << " Error!" << std::endl;
	}
}
#else
void TerminalServer::createScenario(long sce_id)
{
//	int			sysRet;

	m_Incoming		= 0;
	m_Incoming2		= 0;
	m_PA10Waiting	= 0;
	m_scenarioList.clear();
/*
	if(sce_id & 2){		// 案内
		std::cout << "シナリオ作成 案内" << std::endl;
		// 案内要求			( 11 )
		m_scenarioList.push_back(11);
		// 案内終了待機		( 91 )
		if(sce_id & 4){
			// 連続サービスの場合
			m_scenarioList.push_back(95);
		}
		else{
			m_scenarioList.push_back(91);
		}
	}
*/
	if(sce_id & 4){		// 給仕
		std::cout << "シナリオ作成 給仕" << std::endl;
		// 給仕開始要求		( 21 )
		m_scenarioList.push_back(21);
		// RH到着待機		( 95 )
		m_scenarioList.push_back(95);
		// PA10積載開始要求	( 41 )
		m_scenarioList.push_back(41);
		// 積載終了待機		( 92 )
		m_scenarioList.push_back(92);
		// 給仕移動要求		( 22 )
		m_scenarioList.push_back(22);
		// RH到着待機		( 91 )
		m_scenarioList.push_back(91);
		// 受取待機			( 93 )
		m_scenarioList.push_back(93);
		// RH帰還要求		( 51 )
		m_scenarioList.push_back(51);
	}
	if(sce_id & 8){		// 廃棄
		std::cout << "シナリオ作成 廃棄" << std::endl;
		// 廃棄開始要求		( 31 )
		m_scenarioList.push_back(31);
		// RH到着待機		( 91 )
		m_scenarioList.push_back(91);
		// 空き缶積載待機	( 94 )
		m_scenarioList.push_back(94);
		// 廃棄実行要求		( 32 )
		m_scenarioList.push_back(32);
		// 廃棄終了待機		( 95 )
		m_scenarioList.push_back(95);
		// RH帰還要求		( 51 )
		m_scenarioList.push_back(51);
	}
	m_scenarioList.push_back(999);
/*
  void RHGuideReq(in Position2D entrance, in Position2D room, in string name);
  void RHDrinkServeReq(in Position2D visitor, in Position2D warehouse, in string name);
  void RHDrinkTransportReq(in Position2D dist, in string name);
  void RHGarbageCollectReq(in Position2D dist, in string name);
  void RHGarbageDumpReq(in Position2D dumpsite, in string name);
 */
}
#endif

void TerminalServer::runScenario()
{
	long			phase_num;
	static bool 	dbg_msg = true;
/*
	if(m_SystemError == true){
		// システムエラーが発生した
		m_scenarioList.clear();
		m_Incoming = 0;
		m_Incoming2 = 0;
		m_PA10Waiting = 0;
		m_ServiceState = 1;
		std::cout << "***************************************" << std::endl;
		std::cout << "**  システムエラーが発生しました       **" << std::endl;
		std::cout << "***************************************" << std::endl;
		usleep(500000);
		return;
	}
*/
	if(m_scenarioList.size() == 0){
		usleep(500000);
		return;
	}

#ifdef SCENARIO_FILE
	ScenarioInfo	scInfo;

	scInfo = m_scenarioList.front();
	phase_num = scInfo.commandID;
#else
	phase_num = m_scenarioList.front();
#endif

	// 実行コマンドは待機系ではない・・・
	if(phase_num < 90){
		if(m_ScenarioStatus == 2){
			// 一時停止中
			usleep(500000);
			return;
		}
		m_scenarioList.pop_front();
	}
	switch(phase_num){
		case 11:			// 案内要求			( 11 )
			/*
			 * entrance	: 入口
			 * room		: 会議室
			 */
			std::cout << "シナリオ実行 案内要求" << std::endl;
			{
				Position2D entrance, room;
/*
				entrance.x		= m_LandmarkList["入口"].XPos;
				entrance.y		= m_LandmarkList["入口"].YPos;
				entrance.theta	= m_LandmarkList["入口"].Omega;

				room.x		= m_LandmarkList["会議室"].XPos;
				room.y		= m_LandmarkList["会議室"].YPos;
				room.theta	= m_LandmarkList["会議室"].Omega;

				m_ServiceState = 3;
*/
				m_rh_if->RHGuideReq(entrance, room, "呼んでみた");

			}
			break;
		case 21:		// 給仕開始要求		( 21 )
			/*
			 * visitor		: 会議室
			 * warehouse	: PA10
			 */
			std::cout << "シナリオ実行 給仕開始要求" << std::endl;
			{
				Position2D visitor, warehouse;
/*
				visitor.x		= m_LandmarkList["会議室"].XPos;
				visitor.y		= m_LandmarkList["会議室"].YPos;
				visitor.theta	= m_LandmarkList["会議室"].Omega;

				warehouse.x		= m_LandmarkList["PA10"].XPos;
				warehouse.y		= m_LandmarkList["PA10"].YPos;
				warehouse.theta	= m_LandmarkList["PA10"].Omega;

				m_ServiceState = 4;
*/
				m_rh_if->RHDrinkServeReq(visitor, warehouse, "呼んでみた");

			}
			break;
		case 22:		// 給仕移動要求		( 22 )
			/*
			 * dist		: 会議室
			 */
			std::cout << "シナリオ実行 給仕移動要求" << std::endl;
			{
				Position2D dist;
/*
				dist.x		= m_LandmarkList["会議室"].XPos;
				dist.y		= m_LandmarkList["会議室"].YPos;
				dist.theta	= m_LandmarkList["会議室"].Omega;

				m_ServiceState = 4;
*/
				m_rh_if->RHDrinkTransportReq(dist, "呼んでみた");

			}
			break;
		case 31:		// 廃棄開始要求		( 31 )
			/*
			 * dist		: 会議室
			 */
			std::cout << "シナリオ実行 廃棄開始要求" << std::endl;
			{
				Position2D dist;
/*
				dist.x		= m_LandmarkList["会議室"].XPos;
				dist.y		= m_LandmarkList["会議室"].YPos;
				dist.theta	= m_LandmarkList["会議室"].Omega;

				m_ServiceState = 5;

				std::cout << dist.x << "," << dist.y << "," << dist.theta << std::endl;
*/
				m_rh_if->RHGarbageCollectReq(dist, "呼んでみた");

			}
			break;
		case 32:		// 廃棄実行要求		( 32 )
			/*
			 * dumpsite		: ゴミ箱
			 */
			std::cout << "シナリオ実行 廃棄実行要求" << std::endl;
			{
				Position2D dumpsite;
/*
				dumpsite.x		= m_LandmarkList["ゴミ箱"].XPos;
				dumpsite.y		= m_LandmarkList["ゴミ箱"].YPos;
				dumpsite.theta	= m_LandmarkList["ゴミ箱"].Omega;
				m_ServiceState = 5;
*/
				m_rh_if->RHGarbageDumpReq(dumpsite, "呼んでみた");

			}
			break;
		case 41:		// PA10積載開始要求	( 41 )
			/*
			 * PA10に積載開始を要求する
			 */
			std::cout << "シナリオ実行 PA10積載開始要求" << std::endl;
			{
//				m_ServiceState = 4;
#ifdef SCENARIO_FILE
				std::cout << scInfo.param[0] << std::endl;
				m_DrinkNum = string_to_int(scInfo.param[0]);
#endif
				if(m_DrinkNum > 4)	m_DrinkNum = 4;
				if(m_DrinkNum < 1)	m_DrinkNum = 1;
std::cout << "drink = " << m_DrinkNum << std::endl;
				m_pa10if->PA10DrinkServeReq(m_DrinkNum);
			}
			break;
		case 51:		// RH帰還要求
			std::cout << "シナリオ実行 RH帰還要求" << std::endl;
			{
				Position2D base;
/*
				base.x		= m_LandmarkList["ゴミ箱"].XPos;
				base.y		= m_LandmarkList["ゴミ箱"].YPos;
				base.theta	= m_LandmarkList["ゴミ箱"].Omega;
*/
				m_rh_if->RHReturn(base);
			}
			break;
		case 91:		// 廃棄終了待機		( 91 )	// RH到着待機
			if(dbg_msg){
				std::cout << "シナリオ実行 RH移動終了待機" << std::endl;
				dbg_msg = false;
			}
			if( m_Incoming == 1 ){
				// フェーズスタックから取り出し
				m_scenarioList.pop_front();
				m_Incoming = 0;
				m_Incoming2 = 1;

				dbg_msg = true;
			}
			break;
		case 92:		// 積載終了待機		( 92 )	// PA10積載終了待機
			if(dbg_msg){
				std::cout << "シナリオ実行 積載終了待機" << std::endl;
				dbg_msg = false;
			}

			if( m_PA10Waiting == 1 ){
				// フェーズスタックから取り出し
				m_scenarioList.pop_front();
				m_PA10Waiting = 0;

				dbg_msg = true;
			}
			break;
		case 93:		// 受取待機			( 93 )	// お茶受取待機
			if(dbg_msg){
				std::cout << "シナリオ実行 ユーザー操作待機" << std::endl;
				dbg_msg = false;
			}
			if(m_UserOperation == true){
				// フェーズスタックから取り出し
				m_UserOperation = false;
				m_scenarioList.pop_front();

//				m_ServiceState = 1;
				m_Incoming2 = 0;
				dbg_msg = true;
			}
			break;
		case 94:		// 空き缶積載待機	( 94 )	// 空き缶積載待機
			if(dbg_msg){
				std::cout << "シナリオ実行 ユーザー操作待機" << std::endl;
				dbg_msg = false;
			}
			if(m_UserOperation == true){
				// フェーズスタックから取り出し
				m_UserOperation = false;
				m_scenarioList.pop_front();
				m_Incoming2 = 0;

				dbg_msg = true;
			}
			break;
		case 95:		// RH到着待機
			if(dbg_msg){
				std::cout << "シナリオ実行 RH到着待機" << std::endl;
				dbg_msg = false;
			}
			if( m_Incoming == 1 ){
				// フェーズスタックから取り出し
				m_scenarioList.pop_front();
				m_Incoming = 0;

				dbg_msg = true;
			}
			break;
		case 999:
				std::cout << "シナリオ終了" << std::endl;
				m_scenarioList.pop_front();

				m_ScenarioStatus = 0;
//				m_ServiceState = 1;
			break;
		default:
			break;
	}
	usleep(100000);
}
#endif

void TerminalServer::VisitorProc(QueryInfo *pQueryInfo)
{
	std::string StatusStr;
	std::string	Responder;
	std::string	msgBody;
	std::vector<std::string>	toMember;

	m_Visitor = false;

	// 来訪者チケット発行？？
	VisitorTicket	ticket;

	ticket.m_Name	= pQueryInfo->GetQueryPair("Name");
	ticket.m_Company	= pQueryInfo->GetQueryPair("Company");
	ticket.m_PersonInCharge	= pQueryInfo->GetQueryPair("PIC");
	// 訪問先担当者の指定は漢字とカタカナの二つがある？？
	ticket.m_Purpose		= pQueryInfo->GetQueryPair("Purpose");
	// 来訪者ファイルを作成する
	m_TickerNum = ticket.writeData(m_TickerNum);
	m_TicketList[ticket.m_VisitNumber] = ticket;
	m_TickerNum++;

	// ファイル作成が完了したら、gTalkにメッセージを発行する
	// メッセージの形式は・・・
	/*
	 * お客様がお見えになりました。
	 * 来訪者情報確認サイトにアクセスして対応方法の入力をお願いします。
	 * 来訪者情報確認URL : http://hostname/cgi-bin/xxxxxx.cgi?VisitorNumber=XXXX
	 */
	{
		// ボディ部分を事前に組み上げておく
		msgBody = msgBody + "\" body=\"お客様がお見えになりました。\n";
		msgBody = msgBody + "来訪者情報確認サイトにアクセスして対応方法の入力をお願いします。\n";
		msgBody = msgBody + "来訪者情報確認URL : http://" + m_HttpServerIP + ":6809/cgi-bin/terminalServer.cgi?";
		msgBody = msgBody + "Function=FrontDesk";
		msgBody = msgBody + "&VisitorNumber=" + ticket.m_VisitNumber;
		/*
		msgBody = msgBody + "&Responder=" + ticket.m_PersonInCharge;
		msgBody = msgBody + "\" />";
		*/
	}

	// 用件確認
	if(ticket.m_Purpose.compare("打ち合わせ") != 0){
		// 配達・清掃
		// 宛先は受付担当者
		if(m_MemberInfo.GetFrontDeskPIC().m_Available){
			toMember.push_back(m_MemberInfo.GetFrontDeskPIC().m_MailAddr);
std::cout << "Data-1 " << m_MemberInfo.GetFrontDeskPIC().m_MailAddr << std::endl;
		}
		else{
			// 不在なのでその他全員を宛先にする
			toMember = m_MemberInfo.GetAvailableList();
		}
	}
	else{
		// 打合せの場合、宛先は指定担当者
		if(m_MemberInfo[ticket.m_PersonInCharge].m_MailAddr.length() != 0)	{
			// 指定された担当者が在籍しているか確認する
			if(m_MemberInfo[ticket.m_PersonInCharge].m_Available){
				toMember.push_back(m_MemberInfo[ticket.m_PersonInCharge].m_MailAddr);
std::cout << "Data-3 " << m_MemberInfo[ticket.m_PersonInCharge].m_MailAddr << std::endl;
			}
			else{
				msgBody = msgBody + "\" body=\"" + ticket.m_PersonInCharge;
				msgBody = msgBody + "宛にお客様がお見えになりましたが、不在です。\n";
				msgBody = msgBody + "来訪者情報確認サイトにアクセスして対応方法の入力をお願いします。\n";
				msgBody = msgBody + "来訪者情報確認URL : http://" + m_HttpServerIP + ":6809/cgi-bin/terminalServer.cgi?";
				msgBody = msgBody + "Function=FrontDesk";
				msgBody = msgBody + "&VisitorNumber=" + ticket.m_VisitNumber;

				// 不在なので受付担当をチェック
				if(m_MemberInfo.GetFrontDeskPIC().m_Available){
					toMember.push_back(m_MemberInfo.GetFrontDeskPIC().m_MailAddr);
std::cout << "Data-4 " << m_MemberInfo.GetFrontDeskPIC().m_MailAddr << std::endl;
				}
				else{
					// 不在なのでその他全員を宛先にする
std::cout << "Data-5 " << std::endl;
					toMember = m_MemberInfo.GetAvailableList();
				}
			}
		}
		else{
			// 存在しない担当を指定された
			if(m_MemberInfo.GetFrontDeskPIC().m_Available){
				toMember.push_back(m_MemberInfo.GetFrontDeskPIC().m_MailAddr);
std::cout << "Data-6 " << m_MemberInfo.GetFrontDeskPIC().m_MailAddr << std::endl;
			}
			else{
				// 不在なのでその他全員を宛先にする
std::cout << "Data-7 " << std::endl;
				toMember = m_MemberInfo.GetAvailableList();
			}
			// メッセージボディを変更
			msgBody = msgBody + "\" body=\"不審な方がお見えです。\n";
			msgBody = msgBody + "来訪者情報確認サイトにアクセスして対応方法の入力をお願いします。\n";
			msgBody = msgBody + "来訪者情報確認URL : http://" + m_HttpServerIP + ":6809/cgi-bin/terminalServer.cgi?";
			msgBody = msgBody + "Function=FrontDesk";
			msgBody = msgBody + "&VisitorNumber=" + ticket.m_VisitNumber;
			/*
			msgBody = msgBody + "&Responder=" + ticket.m_PersonInCharge;
			msgBody = msgBody + "\" />";
			*/
		}
	}
	std::cout << msgBody << std::endl;

	// メッセージヘッダ組み立て＆送信
	for(std::vector<std::string>::iterator it = toMember.begin(); it != toMember.end(); it++){
		Responder = "&Responder=" + m_MemberInfo.GetNameFromMailAddr(*it);

		StatusStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		StatusStr = StatusStr + "<message to=\"";
		StatusStr = StatusStr + *it;
		StatusStr = StatusStr + msgBody;
		StatusStr = StatusStr + Responder;
		StatusStr = StatusStr + "\" />";
		// GTalkにメッセージを送信する
		if(StatusStr.length() > 0){
			m_send_gtalk_msg.data = StatusStr.c_str();
			m_send_gtalk_msgOut.write();
			std::cout << StatusStr << std::endl;
		}
	}

	// タイマー作動
}

/*
RTC::ReturnCode_t TerminalServer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TerminalServer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TerminalServer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TerminalServer::onStateUpdate(RTC::UniqueId ec_id)
{
	printf("onStateUpdate\n");
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TerminalServer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

bool TerminalServer::SetMemberStatus(std::string member_addr, bool available)
{
	return m_MemberInfo.setAvailable(member_addr, available);
}

bool TerminalServer::AddCandidate(const std::string name, const float score)
{
	float	dataScore;
	bool	result = true;

	if(m_CandidateStage != 9)
		dataScore = m_JuliusScore;
	else
		dataScore = m_CommandScore;

	if(score > dataScore)
		m_CandidateList.push_back(name);
	return result;
}

long TerminalServer::SetTerminalState(long src, long state, std::string state_str)
{
std::cout << src <<  " : ステータス = " << state << " (" << state_str << ")" << std::endl;
	long old = 0;

	if(state == 0){
		switch(src){
			case 3:
				m_Incoming = 1;
				break;
			case 4:
				m_PA10Waiting = 1;
				break;
		}
	}
	/*
	if(state < 0){
		m_SystemError = true;
	}
	else{
		m_SystemError = false;
	}
	*/
	return old;
}

extern "C"
{

  void TerminalServerInit(RTC::Manager* manager)
  {
    coil::Properties profile(terminalserver_spec);
    manager->registerFactory(profile,
                             RTC::Create<TerminalServer>,
                             RTC::Delete<TerminalServer>);
  }

};
