#include <error.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <iostream>
#include <sstream>
#include <strstream>
#include <fstream>
#include "Tools.h"
#include "TextData.h"
#include "TSProperty.h"
#include "UserMemberData.h"

VisitorTicket::VisitorTicket()
{
	time_t pCurTime;

	m_Name				= "";
	m_Company			= "";
	m_PersonInCharge	= "";
	m_Purpose			= "";
	m_Closed			= false;
	m_VisitTime			= time(&pCurTime);
}

bool VisitorTicket::readData(const std::string TicketName)
{
	bool		result = true;
	int			fd;
	ssize_t		readSize;
	std::string	fileName(VISIT_DATA_FOLDER);
	VisitorTicket::RecordData	recordInfo;

	memset(&recordInfo, 0, sizeof(recordInfo));

	m_VisitNumber = TicketName;
	// データファイルを追加モードで開く
	fileName = VISIT_DATA_FOLDER + m_VisitNumber + ".vtf";
	fd = open(fileName.c_str(), O_RDONLY);

	if(fd != -1){
		// データを書き出す
		readSize = read(fd, &recordInfo, sizeof(recordInfo));
		// データを閉じる
		close(fd);

		// データを詰め替える
		m_VisitNumber		= recordInfo.lpszNumber;
		m_Name				= recordInfo.lpszName;
		m_Company			= recordInfo.lpszCompany;
		m_PersonInCharge	= recordInfo.lpszPIC;
		m_ServedName		= recordInfo.lpszServed;
		m_Purpose			= recordInfo.lpszPurpose;
		m_VisitTime			= recordInfo.visitTime;
		m_ExitTime			= recordInfo.exitTime;
	}
	else{
		result = false;
	}
	return result;
}

unsigned int VisitorTicket::writeData(const unsigned int number)
{
	char		numExt[12];
	bool		result = true;
	int			fd = -1;
	unsigned int	num = number;
	ssize_t		writeSize;
	std::string	fileName(VISIT_DATA_FOLDER);
	std::string	numStr;
	VisitorTicket::RecordData	recordInfo;

	// データファイルを追加モードで開く
	errno = EEXIST;
	for(fd = -1; (fd == -1) && (errno == EEXIST); ){
		memset(numExt, 0, sizeof(numExt));
		sprintf(numExt, "_%d", num);
		numStr = numExt;
		m_VisitNumber = VISIT_TICKET_NUMBER + GetDateTimeNum(&m_VisitTime) + numStr;
		fileName = VISIT_DATA_FOLDER + m_VisitNumber + ".vtf";
		fd = open(fileName.c_str(), O_CREAT | O_EXCL | O_WRONLY, 0666);
		num++;
	}
	if((fd != -1) && ((errno == 0) || (errno == 17))){
		// データを詰め替える
		memset(&recordInfo, 0, sizeof(recordInfo));
		strcpy(recordInfo.lpszNumber,	m_VisitNumber.c_str());
		strcpy(recordInfo.lpszName,		m_Name.c_str());
		strcpy(recordInfo.lpszCompany,	m_Company.c_str());
		strcpy(recordInfo.lpszPIC,		m_PersonInCharge.c_str());
		strcpy(recordInfo.lpszServed,	m_ServedName.c_str());
		strcpy(recordInfo.lpszPurpose,	m_Purpose.c_str());
		recordInfo.visitTime		= m_VisitTime;
		recordInfo.exitTime			= m_ExitTime;

		// データを書き出す
		writeSize = write(fd, &recordInfo, sizeof(recordInfo));
		// データを閉じる
		close(fd);
	}
	else{
		std::cout << errno << std::endl;
		result = false;
	}
	return result;
}

int MemberInfo::readData(std::string dataFileName)
{
	TS_Property	systemProperty;
	int			result = 0;

//	if(dataFileName.length() == 0){
	dataFileName = systemProperty[dataFileName];
	// データファイルが存在しているか確認し、存在している場合には読み込み処理を行う
	if(access(dataFileName.c_str(), F_OK) == 0){
		// ファイルを開いてデータを読み込む
		// 氏名、仮名、名字、仮名、メールアドレス、受付担当フラグ　1行1レコード
		TextDataFile	textData;
		field_list		recordData;
		record_it		recData;

		textData.openFile(dataFileName);
		if(textData.readData()){
			recordData.clear();
			recordData = textData.getFields();

			recData = textData.getTopRecord();
			for(; recData != (record_it)NULL; ){
				// レコード情報退避？？
				MemberInfo::Data		InfoData;
				field_data			recVal;

				recordData = textData.getRecord();
				// 最低限の情報はあるらしい
				InfoData.m_Name			= recordData[0];
				InfoData.m_Kana			= recordData[1];
				InfoData.m_ShortName	= recordData[2];
				InfoData.m_ShortKana	= recordData[3];
				InfoData.m_MailAddr		= recordData[4];
				InfoData.m_Available	= true;
#ifdef _DEBUG
				std::cout << InfoData.m_ShortName << " 追加しました。";
#endif
				if(recordData.size() > 5){
					if(recordData[5][0] == '1'){
						InfoData.m_Default	= true;
						m_FrontDeskPIC		= InfoData.m_ShortName;
#ifdef _DEBUG
						std::cout << " 受付担当デス。";
#endif
					}
				}
#ifdef _DEBUG
						std::cout << std::endl;
#endif
				m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));
				recData = textData.getNextRecord();
			}
		}
#if 0
		char		lineData[1024];
		int			line = 0;
		std::string	data;
		std::string::size_type index;

		// ファイルストリームを作成する
		std::ifstream ifs(dataFileName.c_str(), std::ios::in);

		if(ifs){
			for(line = 0; !ifs.eof(); line++){
				memset(lineData, 0, sizeof(lineData));
				ifs.getline(lineData, sizeof(lineData));
				data = lineData;
				if(data.length() > 1){
					// コメント行は削除する
					index = data.find(";");
					if(index != std::string::npos){
						data = data.substr(0, index);
					}
					index = data.find("#");
					if(index != std::string::npos){
						data = data.substr(0, index);
					}
					if(data.length() > 1){
						std::vector<std::string>	recordData;
						// 空白行は削除する
						// レコードを取得する
						recordData = string_token(data, ",");
						if(recordData.size() >= 5){
							MemberInfo::Data		InfoData;

							// 最低限の情報はあるらしい
							InfoData.m_Name			= recordData[0];
							InfoData.m_Kana			= recordData[1];
							InfoData.m_ShortName	= recordData[2];
							InfoData.m_ShortKana	= recordData[3];
							InfoData.m_MailAddr		= recordData[4];
#ifdef _DEBUG
							std::cout << InfoData.m_ShortName << " 追加しました。";
#endif
							if(recordData.size() > 5){
								if(recordData[5][0] == '1'){
									InfoData.m_Default	= true;
									m_FrontDeskPIC		= InfoData.m_ShortName;
#ifdef _DEBUG
									std::cout << " 受付担当デス。";
#endif
								}
							}
#ifdef _DEBUG
									std::cout << std::endl;
#endif
							InfoData.m_Available	= true;
							m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));
						}
					}
				}
			}
		}
#endif
	}
#if 0
	else{
		MemberInfo::Data		InfoData;
		// ダミーデータを入れる
		InfoData.m_Name		= "津幡善信";
		InfoData.m_Kana		= "ツバタヨシノブ";
		InfoData.m_ShortName		= "津幡";
		InfoData.m_ShortKana		= "ツバタ";
		InfoData.m_MailAddr = "yotubata@fsi.co.jp";
		InfoData.m_Default	= true;
		InfoData.m_Available = true;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));
		m_FrontDeskPIC = InfoData.m_ShortName;

		InfoData.m_Name		= "二宮恒樹";
		InfoData.m_Kana		= "ニノミヤツネキ";
		InfoData.m_ShortName		= "二宮";
		InfoData.m_ShortKana		= "ニノミヤ";
		InfoData.m_MailAddr = "tnninomi@fsi.co.jp";
		InfoData.m_Default	= false;
		InfoData.m_Available = true;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));

		InfoData.m_Name		= "小島幸也";
		InfoData.m_Kana		= "コジマユキヤ";
		InfoData.m_ShortName		= "小島";
		InfoData.m_ShortKana		= "コジマ";
		InfoData.m_MailAddr = "kojimayk@fsi.co.jp";
		InfoData.m_Default	= false;
		InfoData.m_Available = true;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));

		InfoData.m_Name		= "杉山彰啓";
		InfoData.m_Kana		= "スギヤマアキヒロ";
		InfoData.m_ShortName		= "杉山";
		InfoData.m_ShortKana		= "スギヤマ";
		InfoData.m_MailAddr = "sugiyaa@fsi.co.jp";
		InfoData.m_Default	= false;
		InfoData.m_Available = true;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));

		InfoData.m_Name		= "小笠原哲也";
		InfoData.m_Kana		= "オガサワラテツヤ";
		InfoData.m_ShortName		= "小笠原";
		InfoData.m_ShortKana		= "オガサワラ";
		InfoData.m_MailAddr = "ogasawar@fsi.co.jp";
		InfoData.m_Default	= false;
		InfoData.m_Available = true;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));

		InfoData.m_Name		= "テスト";
		InfoData.m_Kana		= "テスト";
		InfoData.m_ShortName		= "テスト";
		InfoData.m_ShortKana		= "テスト";
		InfoData.m_MailAddr = "yotubata@fsi.co.jp";
		InfoData.m_Default	= false;
		InfoData.m_Available = false;
		m_List.insert(MEMBER_INFO_VAL(InfoData.m_ShortName, InfoData));
	}
#endif
	return result;
}

bool MemberInfo::setAvailable(std::string mail_addr, bool available)
{
	bool			result = false;
	MemberInfo::Data target;

	target = findMember(mail_addr);
	if(target.m_Kana.length() > 0){
		m_List[target.m_Kana].m_Available = available;
		result = true;
	}

	return result;
}

MemberInfo::Data MemberInfo::findMember(std::string mail_addr)
{
	MemberInfo::Data	result;

	for(MEMBER_INFO_IT it = m_List.begin(); it != m_List.end(); it++){
		if(it->second.m_MailAddr == mail_addr){
			result = it->second;
			break;
		}
	}
	return result;
}

std::vector<std::string> MemberInfo::GetAvailableList()
{
	std::vector<std::string>	result;

std::cout << "Data-2 " << std::endl;

	for(MEMBER_INFO_IT it = m_List.begin(); it != m_List.end(); it++){
		if(it->second.m_Available){
			result.push_back(it->second.m_MailAddr);
std::cout << "Data-2 " << it->second.m_MailAddr << std::endl;
		}
#if 1
		else{
			std::cout << "Data-2' " << it->second.m_MailAddr << std::endl;
		}
#endif
	}
	return result;
}

std::string MemberInfo::GetNameFromMailAddr(std::string mailAddr)
{
	MemberInfo::Data	result;

	result = findMember(mailAddr);

	return result.m_ShortName;
}


MemberInfo::Data MemberInfo::operator[](std::string name)
{
	MEMBER_INFO_IT it = m_List.find(name);

	if(it == m_List.end()){
		// 存在しなかったので仮名で探してみる
		return findMemberFromKana(name);
	}
	return m_List[name];
}

MemberInfo::Data MemberInfo::findMemberFromKana(std::string kana)
{
	MemberInfo::Data	result;

	for(MEMBER_INFO_IT it = m_List.begin(); it != m_List.end(); it++){
		if(it->second.m_ShortKana == kana){
			result = it->second;
			break;
		}
	}
	return result;
}
