/*
 * TextData.cpp
 *
 *  Created on: 2010/12/10
 *      Author: yotubata
 */
#include <iostream>
#include <unistd.h>

#include "TextData.h"
#include "Tools.h"

TextDataFile::TextDataFile(const std::string fileName, const bool writeMode, const unsigned long maxRecord)
{
	if(writeMode){
		if(fileName.length() > 0){
			field_list	fields;
			createFile(fileName, fields);
		}
	}
	else{
		if(fileName.length() > 0){
			openFile(fileName);
		}
	}

	buffSize = maxRecord;
}

bool TextDataFile::openFile(const std::string fileName)
{
	bool	result = true;

	// オープン済みか確認する
	if(ifsDataFile.is_open() || ofsDataFile.is_open()){
		// オープン済みなので、ファイル名を比較する
		// 同一名でも開き直す
		/*if(dataFileName != fileName)*/{
			// ファイル名が違うので、以前のは破棄して作り直す
			if(ifsDataFile.is_open())	ifsDataFile.close();
			if(ofsDataFile.is_open())	ofsDataFile.close();
			if(fileName.length() > 0)
				dataFileName = fileName;
			openFile(dataFileName);
		}
	}
	else{
		// まだオープンしてない
		// ファイルの存在を確認し、存在している場合は読み込みで開く
		if(access(fileName.c_str(), F_OK) == 0){
			// ファイルストリームを作成する
			ifsDataFile.open(fileName.c_str(), std::ios::in);
			if(ifsDataFile){
				dataFileName = fileName;
				fieldNameList.clear();
				recordBuff.clear();
			}
			else{
				std::cout << "Read file open error!" << std::endl;
				result = false;
			}
		}
	}
	return result;
}

/*
 * @brief データファイルを作成する
 */
bool TextDataFile::createFile(const std::string fileName, const field_list dataFieldName)
{
	bool	result = false;

	// 既にファイルがないか確認する
	if(access(fileName.c_str(), F_OK) == 0){
		// あるようなので、エラーにする
		std::cout << "File already exist!" << std::endl;
		unlink(fileName.c_str());
//		return false;
	}

	// ファイルストリームを作成する
	ofsDataFile.open(fileName.c_str(), std::ios::out);
	if(ofsDataFile){
		dataFileName = fileName;
//		fieldNameList.clear();
//		recordBuff.clear();
		fieldNameList = dataFieldName;
	}
	else{
		std::cout << "Write file open error!" << std::endl;
		result = false;
	}

	return result;
}

bool TextDataFile::writeFields()
{
	std::string	recordString = "";

	// フィールド名を出力する
	for(str_it it = fieldNameList.begin(); it != fieldNameList.end(); it++){
		recordString = recordString + *it;
		ofsDataFile << recordString;
		recordString = "\t";
	}
	ofsDataFile << std::endl;

	return true;
}

bool TextDataFile::writeRecord()
{
	std::string		recordString = "";
	field_data		r_data;

	for(record_it it = recordBuff.begin(); it != recordBuff.end(); it++){
		r_data = it->second;
		recordString = "";
		// １レコード処理
		for(str_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
			// フィールド分回る
			recordString = recordString + r_data[*fit];
			ofsDataFile << recordString;
			recordString = "\t";
		}
		ofsDataFile << std::endl;
	}

	return true;
}

int TextDataFile::readData()
{
	bool		result = true;
	std::string	recordString;
	field_list	recordData;
	int	index;

	if(!(ifsDataFile.is_open()))	return false;

	// ファイルを読み込んで、フィールド一覧とレコードを作成する
	// 先頭行はフィールド名の列挙
	// 2行目以降がデータとなるが、先頭が「#;」のどれかの文字で始まる場合はコメント行とする
	// ファイルを読み込んで、シナリオリストを生成する
	curRecord = (record_it)NULL;
	while(getline(ifsDataFile, recordString)){
		if(recordString.length() > 1){
			// コメント行は削除する
			index = recordString.find(";");
			if(index == 0){
				continue;
			}
			index = recordString.find("#");
			if(index == 0){
				continue;
			}
			// データあり
			if(recordString.length() > 1){
				recordData = string_token(recordString, "\t");
				// フィールド一覧作成
				if(fieldNameList.size() > 0){
					int		fieldCnt = 0;
					field_data	oneRecord;
					// データ追加
					for(str_it it = recordData.begin(); it != recordData.end(); it++){
						// データ収納
						oneRecord.insert(field_pair(fieldNameList[fieldCnt], *it));
						fieldCnt++;
					}
					recordBuff.insert(record_pair(recordData[0], oneRecord));
				}
				else{
					// フィールド名取得
					for(str_it it = recordData.begin(); it != recordData.end(); it++){
						// データ収納
						if(it->length() > 0)
							fieldNameList.push_back(*it);
					}
				}
			}
		}
	}

/*
	// ダンプ
	std::cout << "ReadData!" << std::endl;
	for(record_it rit = recordBuff.begin(); rit != recordBuff.end(); rit++){
		for(field_it fit = rit->second.begin(); fit != rit->second.end(); fit++){
			std::cout << fit->second;
		}
		std::cout << std::endl;
	}
	std::cout << "ReadData End!" << std::endl;
*/

	return result;
}

bool TextDataFile::addField(std::string fieldName)
{
	fieldNameList.push_back(fieldName);

	return true;
}

int TextDataFile::writeData(const std::string fileName)
{
	std::string	writeFileName = dataFileName;

	if(fileName.length() > 0)
		writeFileName = fileName;

	if(ifsDataFile.is_open())	ifsDataFile.close();

	if(ofsDataFile.is_open()){
		// オープン済みなので、ファイル名を比較する
		if(dataFileName != writeFileName){
			// ファイル名が違うので、以前のは破棄して作り直す
			if(ifsDataFile.is_open())	ifsDataFile.close();
			if(ofsDataFile.is_open())	ofsDataFile.close();
			dataFileName = writeFileName;
			createFile(writeFileName, fieldNameList);
		}
	}
	else{
		createFile(writeFileName, fieldNameList);
	}
	if(fieldNameList.size() > 0){
		writeFields();
		writeRecord();
	}

	return 0;
}

bool TextDataFile::writeRecord(const std::string key, const field_list recordData)
{
	bool	result = true;
	int		fCnt;

	fCnt = 0;
	for(str_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
		// フィールド分回る
		recordBuff[key][*fit] = recordData[fCnt];
		fCnt++;
	}

	return result;
}

std::string TextDataFile::getData(const std::string key, const std::string fieldName)
{
	return recordBuff[key][fieldName];
}

field_list TextDataFile::getFields()
{
	return fieldNameList;
}

field_list TextDataFile::getRecord()
{
	field_list	result;

	for(str_it it = fieldNameList.begin(); it != fieldNameList.end(); it++){
		result.push_back((curRecord->second)[*it]);
	}

	return result;
}

record_it TextDataFile::getTopRecord()
{
	curRecord = recordBuff.begin();

	return curRecord;
}

record_it TextDataFile::getNextRecord()
{
	if(curRecord == (record_it)NULL){
		return getTopRecord();
	}

	curRecord++;
	if(curRecord == recordBuff.end())
		curRecord = (record_it)NULL;

	return curRecord;
}

str_it TextDataFile::getTopField()
{
	curField = fieldNameList.begin();

	return curField;
}

str_it TextDataFile::getNextField()
{
	curField++;

	return curField;
}
