/*
 * TextData.h
 *
 *  Created on: 2010/12/10
 *      Author: yotubata
 */

#ifndef TEXTDATA_H_
#define TEXTDATA_H_

#include <string>
#include <vector>
#include <map>
#include <fstream>


typedef std::vector<std::string>				field_list;
typedef field_list::iterator					str_it;

typedef std::map< std::string, std::string>	field_data;
typedef field_data::value_type					field_pair;
typedef field_data::iterator					field_it;

typedef std::map< std::string, field_data >	record_data;
typedef record_data::value_type				record_pair;
typedef record_data::iterator					record_it;


class TextDataFile
{
public:
	TextDataFile(const std::string fileName = "", const bool writeMode = false, const unsigned long maxRecord = 0);
//	~TextDataFile();

public:
	bool		openFile(const std::string fileName = "");
	bool		createFile(const std::string fileName, const field_list dataFieldName);
	int			readData();
	int			writeData(const std::string fileName = "");
	bool		writeRecord(const std::string key, const field_list recordData);
	std::string getData(const std::string key, const std::string fieldName);
	bool		addField(std::string fieldName);
	record_it	getTopRecord();
	record_it	getNextRecord();
	str_it		getTopField();
	str_it		getNextField();
	field_list	getFields();
	field_list	getRecord();

private:
	bool writeFields();
	bool writeRecord();

public:
	field_data &operator[](std::string key)	{ return recordBuff[key]; };

private:
	std::string		dataFileName;
	std::ifstream	ifsDataFile;
	std::ofstream	ofsDataFile;
	unsigned long	buffSize;
	bool			allReaded;
	record_data		recordBuff;
	record_it		curRecord;
	field_list		fieldNameList;
	str_it			curField;
};

#endif /* TEXTDATA_H_ */
